bmm.em=function(y,k,r,it=500,eps=0.00001,init=NULL,seed=7,pp=gauher(8),maxstep=100)
{

######################################################################################
# y is the data of dimension n x p, with n sample size and p number of variables
# k is the number of groups
# r is the number of latent variables/factors
######################################################################################

library(combinat)
ptm = proc.time()
set.seed(seed)

### inizializzazione

numobs=nrow(y)
p=ncol(y)
lik=-1000000000

if (is.null(init$alpha)) {alpha=matrix(runif(p*(r+1),-1,1),p,r+1)} else alpha=init$alpha
                          
## vincolo per l'identificabilit
if (r>1) for (i in 2:r) alpha[1:(i-1),(i+1)]=0

if (k>1) memb=cutree(hclust(dist(y,"binary"),method="ward"),k) else memb=rep(1,numobs)
if (is.null(init$w)) {  w=table(memb)/numobs
                        w=t(t(w))} else w=init$w

if (is.null(init$mu)) {mu=matrix(runif(k*r,-1,1),k,r)} else mu=init$mu

if (is.null(init$sigma)) { sigma<-array(0,c(k,r,r))
                            for (i in 1:k) sigma[i,,]=0.5*diag(r) } else sigma=init$sigma

out=bmm.em.alg(y,numobs,r,k,p,it,alpha,w,mu,sigma,eps,lik,pp,maxstep)

Ez.y=out$Ez.y
likelihood=out$likelihood
ps.y=out$ps.y
py.s=out$py.s
alpha<-out$alpha
w<-out$w
mu<-out$mu
sigma<-out$sigma
likelihood<-matrix(likelihood[!likelihood==0])
py=out$py
h=p*(r+1)-(r*(r-1)/2)+(r*(r+1)/2)*(k-1)+r*(k-1)+k-1
lik=likelihood[length(likelihood)]
bic=-2*lik+h*log(numobs)
aic=-2*lik+2*h
EN=entr(ps.y)
clc=-2*lik+2*EN
icl.bic<--2*lik+2*EN+h*log(numobs)
criteri=cbind(bic,aic,clc,icl.bic)

if (k>1)  {  index<-(apply(ps.y, 1, order))[k,]} else index<-rep(k,numobs)

output=list(y=y,py=py,index=index,alpha=alpha,lik=likelihood,w=w,mu=mu,k=k,sigma=sigma,ps.y=ps.y,p=p,numobs=numobs,r=r,py.s=py.s,tempo=proc.time()-ptm,aic=aic,bic=bic,clc=clc,Ez.y=Ez.y,pp=pp,criteri=criteri)
invisible(output)
}


gauher <- function(n) {# Gauss-Hermite:  returns x,w so that
#\int_-\infty^\infty exp(-x^2) f(x) dx \doteq \sum w_i f(x_i)
  EPS <- 3.e-14
  PIM4 <- .7511255444649425
  MAXIT <- 10
  m <- trunc((n+1)/2)
  x <- w <- rep(-1,n)
  for (i in 1:m) {
    if (i==1) {
      z <- sqrt(2*n+1)-1.85575*(2*n+1)^(-.16667)
    } else if(i==2) {
      z <- z-1.14*n^.426/z
    } else if (i==3) {
      z <- 1.86*z-.86*x[1]
    } else if (i==4) {
      z <- 1.91*z-.91*x[2]
    } else {
      z <- 2.*z-x[i-2]
    }
    for (its in 1:MAXIT) {
      p1 <- PIM4
      p2 <- 0
      for (j in 1:n) {
        p3 <- p2
        p2 <- p1
        p1 <- z*sqrt(2/j)*p2-sqrt((j-1)/j)*p3
      }
      pp <- sqrt(2*n)*p2
      z1 <- z
      z <- z1-p1/pp
      if(abs(z-z1) <= EPS) break
    }
    x[i] <- z
    x[n+1-i] <- -z
    w[i] <- 2/(pp*pp)
    w[n+1-i] <- w[i]
  }
  list(x=x,w=w)
}


entr<-function(z)
{
numobs<-nrow(z)
numg<-ncol(z)
temp<-0
z<-ifelse(z==0,z+0.000000000000000000000001,z)
for (i in 1:numg) for (j in 1:numobs) temp<-temp+(z[j,i]*log(z[j,i]))
return(-temp)
}

misc=function(classification, truth)
{
    q <- function(map, len, x) {
        x <- as.character(x)
        map <- lapply(map, as.character)
        y <- sapply(map, function(x) x[1])
        best <- y != x
        if (all(len) == 1)
            return(best)
        errmin <- sum(as.numeric(best))
        z <- sapply(map, function(x) x[length(x)])
        mask <- len != 1
        counter <- rep(0, length(len))
        k <- sum(as.numeric(mask))
        j <- 0
        while (y != z) {
            i <- k - j
            m <- mask[i]
            counter[m] <- (counter[m]%%len[m]) + 1
            y[x == name(map)[m]] <- map[[m]][counter[m]]
            temp <- y != x
            err <- sum(as.numeric(temp))
            if (err < errmin) {
                errmin <- err
                best <- temp
            }
            j <- (j + 1)%%k
        }
        best
    }
    if (any(isNA <- is.na(classification))) {
        classification <- as.character(classification)
        nachar <- paste(unique(classification[!isNA]), collapse = "")
        classification[isNA] <- nachar
    }
    MAP <- mapClass(classification, truth)
    len <- sapply(MAP[[1]], length)
    if (all(len) == 1) {
        CtoT <- unlist(MAP[[1]])
        I <- match(as.character(classification), names(CtoT),
            nomatch = 0)
        one <- CtoT[I] != truth
    }
    else {
        one <- q(MAP[[1]], len, truth)
    }
    len <- sapply(MAP[[2]], length)
    if (all(len) == 1) {
        TtoC <- unlist(MAP[[2]])
        I <- match(as.character(truth), names(TtoC), nomatch = 0)
        two <- TtoC[I] != classification
    }
    else {
        two <- q(MAP[[2]], len, classification)
    }
    err <- if (sum(as.numeric(one)) > sum(as.numeric(two)))
        as.vector(one)
    else as.vector(two)
    bad <- seq(along = classification)[err]
    errorRate = length(bad)/length(truth)
    return(errorRate)
}


mapClass=function (a, b)
{
    l <- length(a)
    x <- y <- rep(NA, l)
    if (l != length(b)) {
        warning("unequal lengths")
        return(x)
    }
    aChar <- as.character(a)
    bChar <- as.character(b)
    Tab <- table(a, b)
    Ua <- dimnames(Tab)[[1]]
    Ub <- dimnames(Tab)[[2]]
    aTOb <- rep(list(Ub), length(Ua))
    names(aTOb) <- Ua
    bTOa <- rep(list(Ua), length(Ub))
    names(bTOa) <- Ub
    k <- nrow(Tab)
    Map <- rep(0, k)
    Max <- apply(Tab, 1, max)
    for (i in 1:k) {
        I <- match(Max[i], Tab[i, ], nomatch = 0)
        aTOb[[i]] <- Ub[I]
    }
    if (is.numeric(b))
        aTOb <- lapply(aTOb, as.numeric)
    k <- ncol(Tab)
    Map <- rep(0, k)
    Max <- apply(Tab, 2, max)
    for (j in (1:k)) {
        J <- match(Max[j], Tab[, j])
        bTOa[[j]] <- Ua[J]
    }
    if (is.numeric(a))
        bTOa <- lapply(bTOa, as.numeric)
    list(aTOb = aTOb, bTOa = bTOa)
}
